<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Email\Models\EventEmail;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Actions\AssociateToEvent;
use EventPortal\Domain\Util\Notification\SystemNotification;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Facades\Notification;
use Spatie\Snapshots\MatchesSnapshots;

class EmailSendTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test **/
    public function can_send_an_event_email_to_all_delegates()
    {
        Notification::fake();

        $event = factory(Event::class)->create([
            'name' => 'Hello World!',
        ]);
        factory(EventDelegate::class, 3)->create([
            'event_id' => $event->id,
        ]);
        $email = factory(EventEmail::class)
            ->state('invite')
            ->create([
                'event_id' => $event,
            ]);

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        Notification::assertSentTo(
            [$event->delegates->pluck('user')],
            SystemNotification::class,
            function (SystemNotification $notification) use ($event) {
                return $notification->subject === "You're invited to attend Hello World!"
                    && $notification->data['event']->id === $event->id;
            }
        );
    }

    /** @test **/
    public function emails_are_sent_to_PA_if_PA_field_present()
    {
        Notification::fake();

        $event = factory(Event::class)
            ->state('published')
            ->create();

        $managedDelegate = factory(Delegate::class)
            ->state('snapshot')
            ->create([
                'pa_email' => 'pa@example.com',
            ]);

        (new AssociateToEvent())->execute($event, [
            $managedDelegate->id,
        ]);

        $email = factory(EventEmail::class)
            ->create([
                'event_id' => $event,
            ]);

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        Notification::assertSentTo(
            $managedDelegate->user,
            SystemNotification::class,
            function (SystemNotification $notification, $channels, $user) use ($managedDelegate) {
                return $user->routeNotificationForMail($notification) === [
                    'pa@example.com' => $managedDelegate->name,
                ];
            }
        );
    }

    /** @test **/
    public function emails_are_sent_to_delegate_if_PA_field_empty()
    {
        Notification::fake();

        $event = factory(Event::class)
            ->state('published')
            ->create();

        $standardDelegate = factory(Delegate::class)
            ->state('snapshot')
            ->create([
                'pa_email' => null,
            ]);

        (new AssociateToEvent())->execute($event, [
            $standardDelegate->id,
        ]);

        $email = factory(EventEmail::class)
            ->create([
                'event_id' => $event,
            ]);

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        Notification::assertSentTo(
            $standardDelegate->user,
            SystemNotification::class,
            function (SystemNotification $notification, $channels, $user) use ($standardDelegate) {
                return $user->routeNotificationForMail($notification) === [
                    $standardDelegate->user->email => $standardDelegate->name,
                ];
            }
        );
    }
}
